from nltk import word_tokenize, pos_tag

"""
Simplified CKY parser
Adapted from code by Steven Bird
Last modified: 2019-11-17 by Amir Zeldes
"""

def cky(table, tokens, rev_grammar):
    """Find length n partitions of tokens that have RHS in rules

    Example "The good dog", span size 2:
    [The good] dog
    The [good dog]
    """

    for span in range(2, len(tokens)+1):  # Go over all span sizes 2 and up
        for start in range(len(tokens)+1-span):  # Go down diagonal, choosing a starting point
            end = start + span  # Ending point of span
            for mid in range(start+1, end):  # Loop over all possible inflection points (partitions of span)
                node1, node2 = table[start][mid], table[mid][end]
                if (node1, node2) in rev_grammar:
                    table[start][end] = rev_grammar[(node1, node2)]
    return table

text = "the cat saw the mouse"
tokens = word_tokenize(text)
tagged = pos_tag(tokens)


token_table = []
# Make an empty table
for i in range(len(tokens)):
    # Add a row
    token_table.append([str(i)])
    for j in range(len(tokens)):
        # Add columns
        token_table[i].append("--")

for i in range(len(tokens)):
    token_table[i][i+1] = tagged[i][1]

rev_grammar = {}
rev_grammar[("DT","NN")] = "NP"
rev_grammar[("VBD","NP")] = "VP"
rev_grammar[("VP","NP")] = "VP"
rev_grammar[("NP","VP")] = "S"
# rev_grammar[("VP","NP")] = ["VP","S"] # if we want to support multiple decompositions, we'd need a structure like this

# What would we need to fully support ambiguity, i.e. multiple possible parses for the same sentences?
# Remember co-indexing (or 'color-coding') possible solutions

full_table = cky(token_table, tokens, rev_grammar)

for row in full_table:
    print(row)
